#ifndef NICO_DELTA_CHECKSUM
#define NICO_DELTA_CHECKSUM

action v4sum() {
    bit<16> tmp = 0;

    tmp = tmp + (bit<16>) hdr.ipv4.src_addr[15:0];              // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv4.src_addr[31:16];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv4.dst_addr[15:0];              // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv4.dst_addr[31:16];             // 16 bit

    tmp = tmp + (bit<16>) hdr.ipv4.totalLen -20;                // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv4.protocol;                    // 8 bit

    tmp = ~tmp;
    meta.v4sum = tmp;
}

action v6sum() {
    bit<16> tmp = 0;

    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[15:0];              // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[31:16];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[47:32];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[63:48];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[79:64];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[95:80];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[111:96];            // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.src_addr[127:112];           // 16 bit

    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[15:0];              // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[31:16];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[47:32];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[63:48];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[79:64];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[95:80];             // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[111:96];            // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.dst_addr[127:112];           // 16 bit

    tmp = tmp + (bit<16>) hdr.ipv6.payload_length;              // 16 bit
    tmp = tmp + (bit<16>) hdr.ipv6.next_header;                 // 8 bit

    tmp = ~tmp;
    meta.v6sum = tmp;
}

action delta_prepare()
{
    v4sum();
    v6sum();
}

action delta_udp_from_v4_to_v6()
{
    delta_prepare();
    hdr.udp.checksum = hdr.udp.checksum + meta.v6sum;

    /* prevent negative wrap around that loses 1 */
    if(meta.v4sum > hdr.udp.checksum) {
        bit<16> tmp = 0xffff - meta.v4sum;
        hdr.udp.checksum = hdr.udp.checksum + tmp;
    } else {
        hdr.udp.checksum = hdr.udp.checksum - meta.v4sum;
    }
}

action delta_tcp_from_v4_to_v6()
{
    delta_prepare();
    hdr.tcp.checksum = hdr.tcp.checksum + meta.v6sum;

    /* prevent negative wrap around that loses 1 */
    if(meta.v4sum > hdr.tcp.checksum) {
        bit<16> tmp = 0xffff - meta.v4sum;
        hdr.tcp.checksum = hdr.tcp.checksum + tmp;
    } else {
        hdr.tcp.checksum = hdr.tcp.checksum - meta.v4sum;
    }
}

action delta_ipv4_from_v6_to_v4()
{
    delta_prepare();

    /* we don't have ANY checksum, but tcp or udp: we can
       base on that ones for calculating the diff for IPv4
    */
    hdr.ipv4.checksum = 0;    /* TO BE DONE! */
}


#endif